@@ -194,14 +194,14 @@ sub _postamble {
     my $gendep = File::Spec::Functions::catfile($top, qw(Basic Gen pm_to_blib));
     $pmdep .= " $coredeps $gendep";
   }
-  my $pp_call_arg = _pp_call_arg($mod, $mod, $pref, $callpack, $multi_c);
+  my $pp_call_arg = _pp_call_arg($mod, $mod, $pref, $callpack, $multi_c||'');
   my $install = '';
   if (!$internal) {
     my $oneliner = _oneliner(qq{exit if \$ENV{DESTDIR}; use PDL::Doc; eval { PDL::Doc::add_module(q{$mod}); }});
     $install = qq|\n\ninstall ::\n\t\@echo "Updating PDL documentation database...";\n\t$oneliner\n|;
   }
   my @generanda = "$pref.xs";
-  push @generanda, map "pp-$_.c", _pp_list_functions($src) if $multi_c;
+  push @generanda, map "pp-$_.c", _pp_list_functions($src, $internal) if $multi_c;
 qq|
 
 $pref.pm : $pmdep
@@ -221,21 +221,21 @@ sub pdlpp_postamble_int {
 # same format as pdlpp_postamble_int
 sub pdlpp_postamble {
   my $w = whereami_any();
-  join '', map _postamble($w, 0, @$_[0..3], 0), @_;
+  join '', map _postamble($w, 0, @$_), @_;
 }
 
 my %flist_cache;
 sub _pp_list_functions {
   require File::Spec::Functions;
-  my ($src) = @_;
+  my ($src, $internal) = @_;
   my $abs_src = File::Spec::Functions::rel2abs($src);
   if (!$flist_cache{$abs_src}) {
     my $w = whereami_any();
-    my $typespm = File::Spec::Functions::catfile($w, qw(Core Types.pm));
-    system $^X, "$typespm.PL", $typespm if !-f $typespm;
+    my $typespm = File::Spec::Functions::catfile($w, $internal ? qw(Core) : (), qw(Types.pm));
+    system $^X, "$typespm.PL", $typespm if $internal and !-f $typespm;
     require $typespm;
     local $INC{'PDL/Types.pm'} = 1;
-    require ''.File::Spec::Functions::catfile($w, qw(Gen PP.pm));
+    require ''.File::Spec::Functions::catfile($w, $internal ? qw(Gen) : (), qw(PP.pm));
     $flist_cache{$abs_src} = [ PDL::PP::list_functions($src) ];
   }
   @{ $flist_cache{$abs_src} };
@@ -244,7 +244,7 @@ sub _pp_list_functions {
 sub _stdargs {
   my ($w, $internal, $src, $pref, $mod, $callpack, $multi_c) = @_;
   my @cbase = $pref;
-  push @cbase, map "pp-$_", _pp_list_functions($src) if $multi_c;
+  push @cbase, map "pp-$_", _pp_list_functions($src, $internal) if $multi_c;
   my @cfiles = map "$_.c", @cbase;
   my @objs = map "$_\$(OBJ_EXT)", @cbase;
   (
@@ -270,7 +270,7 @@ sub pdlpp_stdargs_int {
 }
 
 sub pdlpp_stdargs {
-  _stdargs(undef, 0, @{$_[0]}[0..3], 0);
+  _stdargs(undef, 0, @{$_[0]});
 }
 
 # pdlpp_mkgen($dir)
