@@ -102,6 +102,7 @@ void pdl_readdata_vaffine(pdl *it);
   X(dim_checks, void, (pdl_transvtable *vtable, pdl **pdls, \
     pdl_thread *pdlthread, PDL_Indx *creating, PDL_Indx *ind_sizes)) \
   X(type_coerce, void, (pdl_trans *trans)) \
+  X(trans_badflag_from_inputs, char, (pdl_trans *trans)) \
   X(get_convertedpdl, pdl *, (pdl *pdl,int type)) \
   X(make_trans_mutual, void, (pdl_trans *trans)) \
   X(make_physical, void, (pdl *it)) \
@@ -121,6 +122,10 @@ void pdl_readdata_vaffine(pdl *it);
   X(set_datatype, void, (pdl *a, int datatype)) \
   X(hdr_copy, SV *, (SV *hdrp)) \
   X(hdr_childcopy, void, (pdl_trans *trans)) \
+  X(readdata_affine, void, (pdl_trans *trans)) \
+  X(writebackdata_affine, void, (pdl_trans *trans)) \
+  X(affine_new, void, (pdl *par,pdl *child,PDL_Indx offs,SV *dims,SV *incs)) \
+  X(converttypei_new, void, (pdl *par,pdl *child,int type)) \
   X(dump, void, (pdl *it)) \
   X(sever, pdl *, (pdl *a))
 
@@ -145,13 +150,6 @@ PDL_GENERICLIST(X)
 struct Core {
   I32    Version;
   badvals bvals;  /* store the default bad values */
-/* Affine trans. THESE ARE SET IN ONE OF THE OTHER Basic MODULES
-   and not in Core.xs ! */
-  void (*readdata_affine)(pdl_trans *tr);
-  void (*writebackdata_affine)(pdl_trans *tr);
-  void (*affine_new)(pdl *par,pdl *child,PDL_Indx offs,SV *dims,SV *incs);
-  /* Converttype. Similar */
-  void (*converttypei_new)(pdl *par,pdl *child,int type);
 
 #define X(sym, rettype, args) \
   rettype (*sym) args;
@@ -161,5 +159,19 @@ struct Core {
 
 typedef struct Core Core;
 
+#define PDL_DECLARE_PARAMETER(declini, cast, type, flag, name, pdlname) \
+  declini name ## _datap = (cast(PDL_REPRP_TRANS(pdlname, flag))); \
+  declini name ## _physdatap = (cast(pdlname->data)); \
+  (void)name ## _datap; \
+  (void)name ## _physdatap;
+
+#define PDL_DECLARE_PARAMETER_BADVAL(declini, cast, type, flag, name, pdlname) \
+  PDL_DECLARE_PARAMETER(declini, cast, type, flag, name, pdlname) \
+  type name ## _badval = 0; \
+  PDL_Anyval name ## _anyval_badval = PDL_CORE_(get_pdl_badvalue)(pdlname); \
+  (void)name ## _badval; \
+  (void)name ## _anyval_badval; \
+  ANYVAL_TO_CTYPE(name ## _badval, type, name ## _anyval_badval);
+
 /* __PDLCORE_H */
 #endif
