@@ -414,11 +414,12 @@ our @ISA = qw (PDL::PP::Rule);
 # Probably want this directly in the apply routine but leave as is for now
 #
 sub dosubst_private {
-    my ($src,$sname,$name) = @_;
+    my ($src,$sname,$pname,$name) = @_;
     my $ret = (ref $src ? $src->[0] : $src);
     my %syms = (
 		((ref $src) ? %{$src->[1]} : ()),
 		PRIV => sub {return "$sname->$_[0]"},
+		COMP => sub {return "$pname->$_[0]"},
 		CROAK => sub {PDL::PP::pp_line_numbers(__LINE__-1, "PDL->pdl_barf(\"Error in $name:\" $_[0])")},
 		NAME => sub {return $name},
 		MODULE => sub {return $::PDLMOD},
@@ -454,7 +455,7 @@ sub new {
     die "\$target must be a scalar for PDL::PP::Rule::Substitute" if ref $target;
     die "\$condition must be a scalar for PDL::PP::Rule::Substitute" if ref $condition;
 
-    $class->SUPER::new($target, [$condition, "StructName", "Name"],
+    $class->SUPER::new($target, [$condition, "StructName", "ParamStructName", "Name"],
 				  \&dosubst_private);
 }
 
@@ -489,7 +490,6 @@ my @std_childparent = (
 	PARENT_P => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[0]->'.(join ',',@_).")")},
 	CHILD_PTR => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[1])')},
 	PARENT_PTR => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(pdls[0])')},
-	COMP => sub {PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV('.(join ',',@_).")")}
 );
 
 sub get_std_childparent { return @std_childparent; }
@@ -551,7 +551,7 @@ sub subst_makecomp_private {
 		PDL::PP::Rule::Substitute::Usual::get_std_childparent(),
 		($cn ?
 			(('DO'.$which.'DIMS') => sub {PDL::PP::pp_line_numbers(__LINE__, join '',
-				map $$co{$_}->get_malloc("\$PRIV($_)"),
+				map $$co{$_}->get_malloc("\$$which($_)"),
 				    grep $$co{$_}->need_malloc, @$cn)}) :
 			()
 		),
@@ -670,10 +670,13 @@ PDL_COMMENT("   /* Memory access */                                         ")
 #define PDL_XS_INPLACE(in, out) \
     if (in->state & PDL_INPLACE && (out != in)) { \
 	in->state &= ~PDL_INPLACE; PDL_COMMENT("unset") \
-	out = in; PDL_COMMENT("discard output value, leak ?") \
+	out = in; \
 	PDL->SetSV_PDL(out ## _SV,out); \
     }
 
+#define PDL_XS_INPLACE_CHECK(in) \
+    if (in->state & PDL_INPLACE) barf("inplace input but output given");
+
 #define PDL_FREE_CODE(trans, comp_free_code, ntpriv_free_code) \
     PDL_TR_CLRMAGIC(trans); \
     comp_free_code \
@@ -681,20 +684,6 @@ PDL_COMMENT("   /* Memory access */                                         ")
 	ntpriv_free_code \
     }
 
-#define PDL_DECLARE_PARAMETER(declini, cast, type, flag, name, pdlname) \
-  declini name ## _datap = (cast(PDL_REPRP_TRANS(pdlname, flag))); \
-  declini name ## _physdatap = (cast(pdlname->data)); \
-  (void)name ## _datap; \
-  (void)name ## _physdatap;
-
-#define PDL_DECLARE_PARAMETER_BADVAL(declini, cast, type, flag, name, pdlname) \
-  PDL_DECLARE_PARAMETER(declini, cast, type, flag, name, pdlname) \
-  type name ## _badval = 0; \
-  PDL_Anyval name ## _anyval_badval = PDL->get_pdl_badvalue(pdlname); \
-  (void)name ## _badval; \
-  (void)name ## _anyval_badval; \
-  ANYVAL_TO_CTYPE(name ## _badval, type, name ## _anyval_badval);
-
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
@@ -1026,7 +1015,7 @@ sub pp_def {
 	require PDL::PP::Dims;
 	require PDL::PP::CType;
 	require PDL::PP::PDLCode;
-	PDL::PP::load_deftable();
+	PDL::PP::load_deftable() if !$PDL::PP::deftbl;
 	my($name,%obj) = @_;
 	print "*** Entering pp_def for $name\n" if $::PP_VERBOSE;
 	($name, my $fulldoc) = _pp_parsename($name);
@@ -1041,7 +1030,7 @@ sub pp_def {
 	  if exists $obj{Dump} and $obj{Dump} and $::PP_VERBOSE;
 
 	croak("ERROR: No FreeFunc for pp_def=$name!\n")
-	  unless exists $obj{FreeFunc}; # and $obj{FreeFunc};
+	  unless exists $obj{FreeFunc};
 
 	my $ctext = join("\n\n",@obj{'StructDecl','RedoDimsFunc',
 		'ReadDataFunc','WriteBackDataFunc',
@@ -1329,18 +1318,17 @@ sub OtherPars_nft {
 sub wrap_vfn {
   my (
     $code,$rout,$func_header,
-    $all_func_header,$sname,$stype,
+    $all_func_header,$sname,$pname,$ptype,
     $name,
   ) = @_;
   my $str = join "\n", grep $_, $all_func_header, $func_header, $code;
   PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-void $rout(pdl_trans *__tr) {
-  $stype *$sname = ($stype *) __tr;
+void $rout(pdl_trans *$sname) {
+@{[$ptype ? "  $ptype *$pname = $sname->params;" : ""]}
 $str}
 EOF
 }
-
-my @vfn_args_always = qw(_AllFuncHeader StructName StructType);
+my @vfn_args_always = qw(_AllFuncHeader StructName ParamStructName ParamStructType);
 sub make_vfn_args {
   my ($which) = @_;
   ("${which}Func",
@@ -1394,36 +1382,6 @@ sub callPerlInit {
     indent($ret,$ci);
 } #sub callPerlInit()
 
-# abstract the access to the bad value status
-# - means we can easily change the representation without too
-#   many changes
-#
-# it's also used in one place in PP/PDLCode.pm
-# -- there it's hard-coded
-#
-sub set_badflag   { PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(bvalflag) = 1;' . "\n") }
-sub clear_badflag { PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(bvalflag) = 0;' . "\n") }
-sub get_badflag   { PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(bvalflag)') }
-
-sub get_badflag_priv { PDL::PP::pp_line_numbers(__LINE__-1, '$PRIV(bvalflag)') }
-
-sub set_badstate {
-    my $pdl = shift;
-    PDL::PP::pp_line_numbers(__LINE__-1, "\$SETPDLSTATEBAD($pdl)")
-}
-
-sub clear_badstate {
-    my $pdl = shift;
-    PDL::PP::pp_line_numbers(__LINE__-1, "\$SETPDLSTATEGOOD($pdl)")
-}
-
-sub get_badstate {
-    my $pdl = shift;
-    PDL::PP::pp_line_numbers(__LINE__-1, "\$ISPDLSTATEBAD($pdl)")
-}
-
-sub NT2Decls_p {&NT2Decls__({ToPtrs=>1},@_);}
-
 sub NT2Decls__ {
   my($opts,$onames,$otypes) = @_;
   my $decl;
@@ -1437,8 +1395,8 @@ sub NT2Decls__ {
 }
 
 sub NT2Free_p {
-  my($onames,$otypes) = @_;
-  my $decl = join '', map $otypes->{$_}->get_free("\$PRIV($_)",
+  my($onames,$otypes,$symbol) = @_;
+  my $decl = join '', map $otypes->{$_}->get_free("\$$symbol($_)",
     { VarArrays2Ptrs => 1 }), @$onames;
   $decl ? PDL::PP::pp_line_numbers(__LINE__, $decl) : '';
 }
@@ -1701,15 +1659,18 @@ EOD
    # The substituted version should then replace "NS" with "Subd"
    # So: FreeCodeNS -> FreeCodeSubd
 
+   PDL::PP::Rule::Returns->new("StructName", "__privtrans"),
+   PDL::PP::Rule::Returns->new("ParamStructName", "__params"),
+
    PDL::PP::Rule::Croak->new([qw(P2Child GenericTypes)],
        'Cannot have both P2Child and GenericTypes defined'),
    PDL::PP::Rule->new([qw(Pars HaveThreading CallCopy GenericTypes DefaultFlow AllFuncHeader RedoDimsFuncHeader)],
-		      ["P2Child","Name"],
+		      ["P2Child","Name","StructName"],
       sub {
-        my (undef,$name) = @_;
+        my (undef,$name,$sname) = @_;
         ("PARENT(); [oca]CHILD();",0,0,[PDL::Types::ppdefs_all()],1,
-          pp_line_numbers(__LINE__-1,"\tpdl *__it = ((pdl_trans_affine *)(__tr))->pdls[1];\n\tpdl *__parent = __tr->pdls[0];\n"),
-          pp_line_numbers(__LINE__-1,"PDL->hdr_childcopy(__tr);\n"),
+          pp_line_numbers(__LINE__-1,"\tpdl *__it = $sname->pdls[1];\n\tpdl *__parent = $sname->pdls[0];\n"),
+          pp_line_numbers(__LINE__-1,"PDL->hdr_childcopy($sname);\n"),
         );
       }),
 
@@ -1726,7 +1687,6 @@ EOD
    PDL::PP::Rule::Returns->new("ExtraGenericLoops", [],
 		'Sets ExtraGenericLoops to an empty hash if it does not already exist', {}),
 
-   PDL::PP::Rule::InsertName->new("StructType", 'pdl_trans__${name}'),
    PDL::PP::Rule::InsertName->new("VTableName", 'pdl_${name}_vtable'),
 
    PDL::PP::Rule::Returns->new("Priv", "AffinePriv", 'PDL_Indx incs[$CHILD(ndims)];PDL_Indx offs; '),
@@ -1809,13 +1769,10 @@ EOD
       # from copying.
       #                    --CED 27-Jan-2003
       #
-      # sent [EquivCPOffsCode,BadFlag]
-      #
       # this just reverses PARENT & CHILD in the expansion of
       # the $EQUIVCPOFFS macro (ie compared to CodefromEquivCPOffsCode)
       sub {
-        my $good = shift;
-        my $bflag = shift;
+        my ($good, $bflag) = @_;
         my $bad  = $good;
         # parse 'good' code
         $good =~ s/\$EQUIVCPOFFS\(([^()]+),([^()]+)\)/\$PP(PARENT)[$2] = \$PP(CHILD)[$1]/g;
@@ -1832,7 +1789,6 @@ EOD
 
    PDL::PP::Rule::Returns::NULL->new("ReadDataFuncName", "AffinePriv"),
    PDL::PP::Rule::Returns::NULL->new("WriteBackDataFuncName", "AffinePriv"),
-
    PDL::PP::Rule->new("XSBootCode", ["AffinePriv","VTableName"],
       sub {return "   $_[1].readdata = PDL->readdata_affine;\n" .
              "   $_[1].writebackdata = PDL->writebackdata_affine;\n"}),
@@ -1865,8 +1821,6 @@ EOD
 
    PDL::PP::Rule->new(["OtherParNames","OtherParTypes"], ["OtherPars","SignatureObj"], \&OtherPars_nft),
 
-   PDL::PP::Rule::Returns->new("StructName", "__privtrans"),
-
    PDL::PP::Rule->new("NewXSArgs", ["SignatureObj","OtherParNames","OtherParTypes"],
       sub {
         my($sig,$onames,$oobjs) = @_;
@@ -1880,7 +1834,7 @@ EOD
 
    PDL::PP::Rule::Returns->new("PMCode", undef),
 
-   PDL::PP::Rule->new("InplaceCode", ["SignatureObj","Inplace"],
+   PDL::PP::Rule->new(["InplaceCode","InplaceCheck"], ["SignatureObj","Inplace"],
 		      'Insert code (just after HdrCode) to ensure the routine can be done inplace',
       # insert code, after the autogenerated xs argument processing code
       # produced by VarArgsXSHdr and AFTER any in HdrCode
@@ -1911,9 +1865,13 @@ EOD
             unless defined $in;
         confess "ERROR: Inplace does not know name of output ndarray\n"
             unless defined $out;
-        PDL::PP::pp_line_numbers(__LINE__-1, "PDL_XS_INPLACE($in, $out)\n");
+        (
+         PDL::PP::pp_line_numbers(__LINE__-1, "PDL_XS_INPLACE($in, $out)\n"),
+         PDL::PP::pp_line_numbers(__LINE__-1, "PDL_XS_INPLACE_CHECK($in)\n"),
+        );
       }),
    PDL::PP::Rule::Returns::EmptyString->new("InplaceCode", []),
+   PDL::PP::Rule::Returns::EmptyString->new("InplaceCheck", []),
 
    PDL::PP::Rule::Returns::EmptyString->new("HdrCode", [],
 					    'Code that will be inserted at the end of the autogenerated xs argument processing code VargArgsXSHdr'),
@@ -1922,7 +1880,7 @@ EOD
    PDL::PP::Rule::Returns->new("VarArgsXSHdr","GlobalNew",undef),
    PDL::PP::Rule->new("VarArgsXSHdr",
       ["Name","NewXSArgs","SignatureObj","OtherParTypes",
-       "PMCode","HdrCode","InplaceCode","_CallCopy","_Bitwise"],
+       "PMCode","HdrCode","InplaceCode","InplaceCheck","_CallCopy","_Bitwise"],
       'XS code to process arguments on stack based on supplied Pars argument to pp_def; GlobalNew has implications how/if this is done',
       # This subroutine is called when no 'otherpars' exist.
       # This writes an XS header which handles variable argument lists,
@@ -1931,7 +1889,7 @@ EOD
       # The use of 'DO NOT SET!!' looks ugly.
       sub {
         my($name,$xsargs,$sig,$optypes,
-           $pmcode,$hdrcode,$inplacecode,$callcopy,$bitwise) = @_;
+           $pmcode,$hdrcode,$inplacecode,$inplacecheck,$callcopy,$bitwise) = @_;
         # Don't do var args processing if the user has pre-defined pmcode
         return 'DO NOT SET!!' if ($pmcode);
         my $ci = '  ';  # current indenting
@@ -1968,7 +1926,7 @@ EOD
                           # the 'initialize' perl routine from the correct package.
         $ci = '    ';  # Current indenting
         # clause for reading in all variables
-        my $clause1 = ''; $cnt = 0;
+        my $clause1 = $inplacecheck; $cnt = 0;
         foreach my $x (@args) {
             if ($other{$x}) {  # other par
                 $clause1 .= "$ci$x = " . typemap($x, $$optypes{$x}, "ST($cnt)") . ";\n";
@@ -1976,7 +1934,9 @@ EOD
             } elsif ($outca{$x}) {
                 push (@create, $x);
             } else {
-                $clause1 .= "$ci$x = PDL->SvPDLV(ST($cnt));\n" if !$already_read{$x};
+                $clause1 .= "$ci$x = PDL->SvPDLV(".
+		  (($out{$x}||$tmp{$x}) ? "${x}_SV = " : '').
+		  "ST($cnt));\n" if !$already_read{$x};
                 $cnt++;
             }
         }
@@ -2096,17 +2056,6 @@ END
           "void $func_name($longpars)");
       }),
 
-   PDL::PP::Rule->new("NewXSStructInit0",
-		      ["StructName","StructType","VTableName"],
-		      "Rule to create and initialise the private trans structure",
-      sub {
-        my( $sname, $stype, $vtable ) = @_;
-        PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-if (!PDL) croak("PDL core struct is NULL, can't continue");
-$stype *$sname = (void *)PDL->create_trans(&$vtable);
-EOF
-      }),
-
    PDL::PP::Rule->new("NewXSMakeNow", ["SignatureObj"],
       sub {
         my($sig) = @_;
@@ -2133,7 +2082,7 @@ EOF
 
    PDL::PP::Rule->new("NewXSTypeCoerceNS", ["StructName"],
       sub {
-        PDL::PP::pp_line_numbers(__LINE__-1, "PDL->type_coerce((pdl_trans *)$_[0]);");
+        PDL::PP::pp_line_numbers(__LINE__-1, "PDL->type_coerce($_[0]);");
       }),
    PDL::PP::Rule::Substitute::Usual->new("NewXSTypeCoerceSubd", "NewXSTypeCoerceNS"),
 
@@ -2156,7 +2105,7 @@ EOF
    PDL::PP::Rule->new("NewXSRunTrans", ["StructName"], sub {
       my($trans) = @_;
       PDL::PP::pp_line_numbers(__LINE__,
-      "PDL->make_trans_mutual((pdl_trans *)$trans);\n");
+      "PDL->make_trans_mutual($trans);\n");
    }),
 
    PDL::PP::Rule->new(PDL::PP::Code::make_args("Code"),
@@ -2177,30 +2126,41 @@ EOF
 # This makes the objects: type + ...
 #
    PDL::PP::Rule->new(["CompNames","CompObjs"], "Comp", \&OtherPars_nft),
-   PDL::PP::Rule->new("CompiledRepr", ["CompNames","CompObjs"], \&NT2Decls_p),
+   PDL::PP::Rule->new("CompiledRepr", ["CompNames","CompObjs"], sub {NT2Decls__({ToPtrs=>1},@_)}),
    PDL::PP::Rule::MakeComp->new("MakeCompiledReprNS", ["MakeComp","CompNames","CompObjs"],
 				"COMP"),
 
-   PDL::PP::Rule->new("CompFreeCode", ["CompNames","CompObjs"], \&NT2Free_p),
+   PDL::PP::Rule->new("CompFreeCode", ["CompNames","CompObjs"], sub {NT2Free_p(@_,"COMP")}),
 
 # This is the default
 #
    PDL::PP::Rule->new("MakeCompiledReprNS",
-      ["OtherParNames","OtherParTypes","StructName"],
+      ["OtherParNames","OtherParTypes","ParamStructName"],
       sub {
-        my($onames,$otypes,$sname) = @_;
+        my($onames,$otypes,$pname) = @_;
         PDL::PP::pp_line_numbers(__LINE__,
-          join '', map $otypes->{$_}->get_copy("$_","$sname->$_"), @$onames
+          join '', map $otypes->{$_}->get_copy($_,"$pname->$_"), @$onames
         );
       }),
    PDL::PP::Rule->new("CompiledRepr",
       ["OtherParNames","OtherParTypes"],
       sub {NT2Decls__({},@_)}),
-   PDL::PP::Rule->new("CompFreeCode", ["OtherParNames","OtherParTypes"], \&NT2Free_p),
+   PDL::PP::Rule->new("CompFreeCode", ["OtherParNames","OtherParTypes"], sub {NT2Free_p(@_,"COMP")}),
+
+   PDL::PP::Rule->new(["StructDecl","ParamStructType"],
+      ["SignatureObj","CompiledRepr","Name"],
+      sub {
+        my($sig,$comp,$name) = @_;
+        return ('', '') if !$comp;
+        my $npdls = @{ $sig->names };
+        my $ptype = "pdl_params_$name";
+        (PDL::PP::pp_line_numbers(__LINE__-1, qq{typedef struct $ptype {\n$comp} $ptype;}),
+        $ptype);
+      }),
 
    PDL::PP::Rule->new("DefaultRedoDims",
       ["StructName"],
-      sub { 'PDL->default_redodims((pdl_trans *)'.$_[0].');' }),
+      sub { 'PDL->default_redodims('.$_[0].');' }),
 
    PDL::PP::Rule->new("DimsSetters",
       ["SignatureObj"],
@@ -2222,9 +2182,7 @@ EOF
    PDL::PP::Rule::Returns::EmptyString->new("Priv"),
 
    PDL::PP::Rule->new(["PrivNames","PrivObjs"], "Priv", \&OtherPars_nft),
-   PDL::PP::Rule->new("PrivateRepr", ["PrivNames","PrivObjs"], \&NT2Decls_p),
-
-   PDL::PP::Rule->new("NTPrivFreeCode", ["PrivNames","PrivObjs"], \&NT2Free_p),
+   PDL::PP::Rule->new("NTPrivFreeCode", ["PrivNames","PrivObjs"], sub {NT2Free_p(@_,"PRIV")}),
 
    PDL::PP::Rule::Substitute->new("MakeCompiledReprSubd", "MakeCompiledReprNS"),
 
@@ -2240,42 +2198,11 @@ EOF
    PDL::PP::Rule::MakeComp->new("NewXSCoerceMustCompNS", "NewXSCoerceMustSubd", "FOO"),
    PDL::PP::Rule::Substitute->new("NewXSCoerceMustCompSubd", "NewXSCoerceMustCompNS"),
 
-   PDL::PP::Rule->new("NewXSFindBadStatusNS",
-      ["BadFlag","_FindBadStatusCode","NewXSArgs","SignatureObj","OtherParTypes","Name"],
+   PDL::PP::Rule->new("NewXSFindBadStatusNS", "FindBadStatusCode", sub {@_}),
+   PDL::PP::Rule->new("NewXSFindBadStatusNS", ["StructName"],
       "Rule to find the bad value status of the input ndarrays",
-      # checks the input ndarrays to see if the routine
-      # is being any data containing bad values
-      #
-      # if FindBadStatusCode is set, use it,
-      # otherwise create the code automatically.
-      #
-      # - in the automatic code creation,
-      # if $badflag is 0, rather than being undefined, then
-      # we issue a warning if any ndarrays contain bad values
-      # (and set the bvalflag to 0)
       sub {
-        my ( $badflag, $badcode, $xsargs, $sig, $optypes, $name ) = @_;
-        return PDL::PP::pp_line_numbers(__LINE__, $badcode) if defined $badcode;
-        my $clear_bad = clear_badflag();
-        my $set_bad   = set_badflag();
-        my $get_bad   = get_badflag();
-        my $str = PDL::PP::pp_line_numbers(__LINE__-1, $clear_bad);
-        # set the badflag_cache variable if any input ndarray has the bad flag set
-        if (my @bval_in = $sig->names_in) {
-          $str .= PDL::PP::pp_line_numbers __LINE__-1, "int \$BADFLAGCACHE() = " .
-            join('||', map get_badstate($_), @bval_in) .
-            ";\n  if (\$BADFLAGCACHE()) ${set_bad}\n";
-        } else {
-          print "\nNOTE: $name has no input bad ndarrays.\n\n" if $::PP_VERBOSE;
-        }
-        if ( defined($badflag) and $badflag == 0 ) {
-          $str .= "  if ( $get_bad ) {
-          printf(\"WARNING: $name does not handle bad values.\\n\");
-          $clear_bad
-      }\n";
-          print "\nNOTE: $name does not handle bad values.\n\n" if $::PP_VERBOSE;
-        } # if: $badflag
-        $str
+        PDL::PP::pp_line_numbers(__LINE__-1, "char \$BADFLAGCACHE() = PDL->trans_badflag_from_inputs($_[0]);\n");
       }),
 
    PDL::PP::Rule->new("NewXSCopyBadStatusNS",
@@ -2298,7 +2225,7 @@ EOF
         return '' if @{$sig->names} == (my @outs = $sig->names_out); # no input pdls, no badflag copying needed
         PDL::PP::pp_line_numbers(__LINE__-1, join '',
           "if (\$BADFLAGCACHE()) {\n",
-          (map "  " . set_badstate($_) . ";\n", @outs),
+          (map "  \$SETPDLSTATEBAD($_);\n", @outs),
           "}\n");
       }),
 
@@ -2307,12 +2234,23 @@ EOF
    PDL::PP::Rule::Substitute::Usual->new("NewXSFindBadStatusSubd", "NewXSFindBadStatusNS"),
    PDL::PP::Rule::Substitute::Usual->new("NewXSCopyBadStatusSubd", "NewXSCopyBadStatusNS"),
 
+   PDL::PP::Rule->new("NewXSStructInit0",
+		      ["StructName","VTableName","ParamStructName","ParamStructType"],
+		      "Rule to create and initialise the private trans structure",
+      sub {
+        my( $sname, $vtable, $pname, $ptype ) = @_;
+        PDL::PP::pp_line_numbers(__LINE__, <<EOF);
+if (!PDL) croak("PDL core struct is NULL, can't continue");
+pdl_trans *$sname = PDL->create_trans(&$vtable);
+@{[$ptype ? "  $ptype *$pname = $sname->params;" : ""]}
+EOF
+      }),
+
    PDL::PP::Rule->new(["RunFunc"],
       ["RunFuncHdr",
         "NewXSStructInit0",
         "NewXSSetTransPDLs",
         "NewXSFindBadStatusSubd",
-        #     NewXSCopyBadValues,
         #     NewXSMakeNow, # this is unnecessary since families never got implemented
         "NewXSTypeCoerceSubd",
         "NewXSExtractTransPDLs",
@@ -2345,16 +2283,6 @@ EOF
       "Rule to print out XS code when variable argument list XS processing is disabled",
       sub {make_xs_code('CODE:',' XSRETURN(0);',@_)}),
 
-   PDL::PP::Rule->new("StructDecl",
-      ["SignatureObj","CompiledRepr","PrivateRepr","StructType"],
-      sub {
-        my($sig,$comp,$priv,$name) = @_;
-        my $npdls = @{ $sig->names };
-        PDL::PP::pp_line_numbers(__LINE__-1, qq{typedef struct $name {
-PDL_TRANS_START($npdls);
-@{[ join "\n", grep $_, $priv, $comp ]}} $name;});
-      }),
-
    PDL::PP::Rule::MakeComp->new("RedoDims-PostComp",
       ["RedoDims", "PrivNames", "PrivObjs"], "PRIV"),
 
@@ -2397,21 +2325,27 @@ PDL_TRANS_START($npdls);
 
    PDL::PP::Rule::Returns::Zero->new("NoPthread"), # assume we can pthread, unless indicated otherwise
    PDL::PP::Rule->new("VTableDef",
-      ["VTableName","StructType","RedoDimsFuncName","ReadDataFuncName",
+      ["VTableName","ParamStructType","RedoDimsFuncName","ReadDataFuncName",
        "WriteBackDataFuncName","FreeFuncName",
        "SignatureObj","Affine_Ok","HaveThreading","NoPthread","Name",
-       "GenericTypes","IsAffineFlag","ReversibleFlag","DefaultFlowFlag"],
+       "GenericTypes","IsAffineFlag","ReversibleFlag","DefaultFlowFlag",
+       "BadFlag"],
       sub {
-        my($vname,$stype,$rdname,$rfname,$wfname,$ffname,
+        my($vname,$ptype,$rdname,$rfname,$wfname,$ffname,
            $sig,$affine_ok,$havethreading, $noPthreadFlag, $name, $gentypes,
-           $affflag, $revflag, $flowflag) = @_;
+           $affflag, $revflag, $flowflag, $badflag) = @_;
         my ($pnames, $pobjs) = ($sig->names_sorted, $sig->objs);
         my $nparents = 0 + grep {! $pobjs->{$_}->{FlagW}} @$pnames;
         my $aff = ($affine_ok ? "PDL_TPDL_VAFFINE_OK" : 0);
         my $npdls = scalar @$pnames;
         my $join_flags = join(", ",map {$pobjs->{$pnames->[$_]}->{FlagPhys} ?
                                           0 : $aff} 0..$npdls-1) || '0';
-        my $op_flags = $havethreading ? 'PDL_TRANS_DO_THREAD' : '0';
+        my @op_flags;
+        push @op_flags, 'PDL_TRANS_DO_THREAD' if $havethreading;
+        push @op_flags, 'PDL_TRANS_BADPROCESS' if $badflag;
+        push @op_flags, 'PDL_TRANS_BADIGNORE' if defined $badflag and !$badflag;
+        push @op_flags, 'PDL_TRANS_NO_PARALLEL' if $noPthreadFlag;
+        my $op_flags = join('|', @op_flags) || '0';
         my $iflags = join('|', grep $_, $affflag, $revflag, $flowflag) || '0';
         my $gentypes_txt = join(", ", (map PDL::Type->new($_)->sym, @$gentypes), '-1');
         my @realdims = map 0+@{$_->{IndObjs}}, @$pobjs{@$pnames};
@@ -2425,6 +2359,7 @@ PDL_TRANS_START($npdls);
         my $realdim_inds = join(", ", @rd_inds) || '0';
         my @indnames = $sig->ind_names_sorted;
         my $indnames = join(",", map qq|"$_"|, @indnames) || '""';
+        my $sizeof = $ptype ? "sizeof($ptype)" : '0';
         PDL::PP::pp_line_numbers(__LINE__, <<EOF);
 static pdl_datatypes ${vname}_gentypes[] = { $gentypes_txt };
 static char ${vname}_flags[] = {
@@ -2445,10 +2380,9 @@ pdl_transvtable $vname = {
   ${vname}_parflags, ${vname}_partypes,
   ${vname}_realdims_starts, ${vname}_realdims_ind_ids, @{[scalar @rd_inds]},
   @{[scalar @indnames]}, ${vname}_indnames,
-  $noPthreadFlag,
   $rdname, $rfname, $wfname,
   $ffname,
-  sizeof($stype),"$::PDLMOD\::$name"
+  $sizeof,"$::PDLMOD\::$name"
 };
 EOF
       }),
